// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';
import 'dart:io';

import 'package:wyatt_cli_toolbox/src/sequences/sequences.dart';

/// The [frames] field holds each individual frame of an animation, whilst the
/// [interval] represents the sleep time between each frame.
class Indicator {
  
  Indicator({
    required this.name,
    required this.interval,
    required this.frames,
    this.label,
  });

  factory Indicator.fromMap(Map<String, Map<String, Object>> map) {
    final String name = map.keys.first;
    return Indicator(
      name: name,
      // ignore: cast_nullable_to_non_nullable
      interval: map[name]!['interval'] as int,
      // ignore: cast_nullable_to_non_nullable
      frames: List<String>.from(map[name]!['frames'] as List<String>),
    );
  }

  final String name;
  final int interval;
  final List<String> frames;
  final String? label;
  Timer? _timer;


  Map<String, Map<String, Object>> toMap() {
    return <String, Map<String, Object>>{
      name: <String, Object>{
        'interval': interval,
        'frames': frames,
      },
    };
  }

  /// Animate the spinner.
  void animate() {
    stop();
    int frame = 0;
    _timer = Timer.periodic(Duration(milliseconds: interval), (Timer t) {
      Sequences.clearEntireLine();
      frame = (frame + 1) % frames.length;
      if (label != null) {
        stdout.write('${frames[frame]} $label');
      } else {
        stdout.write(frames[frame]);
      }
      Sequences.moveHorizontal(1);
    });
  }

  /// Stop the spinner (but don't kill isolate)
  void stop() {
    Sequences.clearEntireLine();
    _timer?.cancel();
  }

  Indicator copyWith({
    String? name,
    int? interval,
    List<String>? frames,
    String? label,
  }) {
    return Indicator(
      name: name ?? this.name,
      interval: interval ?? this.interval,
      frames: frames ?? this.frames,
      label: label ?? this.label,
    );
  }

  @override
  String toString() =>
      'Indicator(name: $name, interval: $interval, frames: $frames)';
}
